//
//  Shader_ES300_Base.fsh
//  CityBuilder
//
//  Created by Lee Metcalfe on 23/06/2014.
//  Copyright (c) 2011 Midoki Ltd. All rights reserved.
//
//------------------------------------------------------------------------------

// Defines and macros to allow the shader code to be compiled for ES2 and ES3...
#if defined ES_300

#define ATTRIBUTE				in
#define VARYING_IN				in
#define VARYING_OUT				out
#define UNIFORM					uniform
#define FRAGCOLOUR				out_fragColour
#define SAMPLER2D				uniform sampler2D
#define SAMPLER2DSHADOW			uniform mediump sampler2DShadow
#define TEXTURE2D(a,b)			texture(a,b)
#define TEXTURE2DSHADOW(a,b)	texture(a,b.xyz/b.w)

#if defined (ANDROID)
#define TEXTURE2DBIAS(a,b,c)	texture(a,b)
#else
#define TEXTURE2DBIAS(a,b,c)	texture(a,b,c)
#endif

#else

#define ATTRIBUTE				attribute
#define VARYING_IN				varying
#define VARYING_OUT				varying
#define UNIFORM					uniform
#define FRAGCOLOUR				gl_FragColor
#define SAMPLER2D				uniform sampler2D
#define SAMPLER2DSHADOW			uniform sampler2D
#define TEXTURE2D(a,b)			texture2D(a,b)
#define TEXTURE2DSHADOW(a,b)	texture2DProj(a,b).x

#if defined (ANDROID)
#define TEXTURE2DBIAS(a,b,c)	texture2D(a,b)
#else
#define TEXTURE2DBIAS(a,b,c)	texture2D(a,b,c)
#endif

#endif

//------------------------------------------------------------------------------

#if defined (SHADER_LIGHT) || defined (SHADER_VCOL) || defined (SHADER_FIXED_COLOUR)
	VARYING_IN lowp		vec4 varying_rgba;
#endif

#if defined (SHADER_TEXTURE)
	VARYING_IN lowp		vec2 varying_uv0;
	#if defined (SHADER_TEXTURE_MULTI_UV2)
	VARYING_IN lowp		vec2 varying_uv1;
	#endif
	#if defined (SHADER_TEXTURE_UWCAUSTICS)
	VARYING_IN mediump	vec4 varying_causticsUVandFadeValues;
	#endif
#endif

#if defined (SHADER_UWTINT)
	VARYING_IN	mediump	vec4 varying_uwTintColour;
#endif

#if defined (SHADER_SHADOW_RECEIVE)
	VARYING_IN lowp		vec4 varying_shadowAmbient;
	VARYING_IN highp	vec4 varying_shadowUVZW;
#endif

//------------------------------------------------------------------------------

#if defined (SHADER_ADD_COLOUR)
	UNIFORM mediump		vec4 uniform_addColour;
#endif

//------------------------------------------------------------------------------

#if defined (SHADER_TEXTURE)
	SAMPLER2D uniform_texture0;
	#if defined (SHADER_TEXTURE_MULTI)
	SAMPLER2D uniform_texture1;
	#endif
	#if defined (SHADER_TEXTURE_UWCAUSTICS)
	SAMPLER2D uniform_causticsmap;
	#endif
#endif

#if defined (SHADER_SHADOW_RECEIVE)
	SAMPLER2DSHADOW uniform_shadowmap;
#endif

//------------------------------------------------------------------------------

#if defined ES_300
out lowp vec4 out_fragColour;
#endif

//------------------------------------------------------------------------------

void main()
{
#if defined (SHADER_DEPTH_ONLY)

	FRAGCOLOUR = vec4(1.0, 1.0, 1.0, 1.0);

#else
	
#if defined (SHADER_TEXTURE)
	
	#if defined (SHADER_TEXTURE_ALPHA_ONLY)
	lowp float	alpha	= TEXTURE2DBIAS(uniform_texture0, varying_uv0,  -1.0).w;
	lowp vec4	albedo	= vec4(1.0, 1.0, 1.0, alpha);
	#else
	lowp vec4	albedo	= TEXTURE2DBIAS(uniform_texture0, varying_uv0,  -1.0);
	#endif
	
	#if defined (SHADER_TEXTURE_ALPHA_DISCARD)
	if (albedo.w < 0.8)
	{
		discard;
	}
	#endif

	#if defined (SHADER_TEXTURE_MULTI)
		#if defined (SHADER_TEXTURE_MULTI_UV2)
		lowp vec4 texel2 = TEXTURE2DBIAS(uniform_texture1, varying_uv1, -1.0);
		#else
		lowp vec4 texel2 = TEXTURE2DBIAS(uniform_texture1, varying_uv0, -1.0);
		#endif
		
		#if defined (SHADER_TEXTURE_MULTI_MULTIPLY)
		albedo.xyz = (albedo.xyz * texel2.xyz);
		#else
		albedo.xyz = (albedo.xyz * (1.0 - texel2.w)) + (texel2.xyz * texel2.w);
		#endif
	#endif
	
	// SPECIAL: Faded animated caustic effect based on depth below water...
	#if defined (SHADER_TEXTURE_UWCAUSTICS)
		lowp vec4	caustics = TEXTURE2D(uniform_causticsmap, varying_causticsUVandFadeValues.xy);
		lowp float	causticsFade = max((varying_causticsUVandFadeValues.z - varying_causticsUVandFadeValues.w), 0.0);
	#endif
	
#else

	lowp vec4 albedo = vec4(1.0, 1.0, 1.0, 1.0);
	
#endif

#if defined (SHADER_LIGHT) || defined (SHADER_VCOL) || defined (SHADER_FIXED_COLOUR)
	lowp vec4 diffuse = varying_rgba;
#else
	lowp vec4 diffuse = vec4(1.0, 1.0, 1.0, 1.0);
#endif
	
	// SPECIAL: Faded animated caustic effect based on depth below water...
#if defined (SHADER_SHADOW_RECEIVE)
//	mediump float shadow = texture2DProj(uniform_shadowmap, varying_shadowUVZW).x;
	mediump float shadow = TEXTURE2DSHADOW(uniform_shadowmap, varying_shadowUVZW);
	diffuse.xyz = (diffuse.xyz * shadow) + (varying_shadowAmbient.xyz * (1.0 - shadow));
#endif

#if defined (SHADER_TEXTURE)
#if defined (SHADER_TEXTURE_UWCAUSTICS)
	diffuse.xyz = (diffuse.xyz + (caustics.xyz * causticsFade));
#endif
#endif
	
	// SPECIAL: Colourize the texture by combining the intensity of the pixel with the colour...
#if defined (SHADER_VCOL_COLOURIZE)
	lowp float intensity = max(max(albedo.x, albedo.y), albedo.z) * 1.5;
//	lowp vec4 finalRGBA = vec4((intensity * diffuse.xyz), albedo.w);
	lowp vec4 finalRGBA = vec4(intensity, intensity, intensity, albedo.w) * diffuse;
#else
	lowp vec4 finalRGBA = (albedo * diffuse);
#endif
	
	// SPECIAL: Colour tint based on depth below water...
#if defined (SHADER_UWTINT)
	mediump	float fogValue = clamp(varying_uwTintColour.w, 0.0 ,1.0);
	fogValue = (0.2 + clamp((fogValue - 0.2), 0.0, 0.6)) * sign(max(fogValue, 0.0));
	finalRGBA.xyz = ((finalRGBA.xyz * (1.0 - fogValue)) + (varying_uwTintColour.xyz * fogValue));
#endif
	
	// SPECIAL: Added colour tint for unit special effects...
#if defined (SHADER_ADD_COLOUR)
	mediump float intensity = dot(finalRGBA.xyz, vec3(0.3, 0.5, 0.2));
	mediump vec3 tintedRGB = (uniform_addColour.xyz + vec3(intensity, intensity, intensity));
	finalRGBA.xyz = (finalRGBA.xyz * (1.0 - uniform_addColour.w)) + (tintedRGB * uniform_addColour.w);
#endif
	
	FRAGCOLOUR = finalRGBA;

#endif
}

//------------------------------------------------------------------------------
